// Scilab code for 
// The paradoxes hidden behind the Droop model highlighted by a metabolic approach
// by C Baroukh, F Mairet and O Bernard 
//
// Adapted from Baroukh et al (Plos One, 2014)
//
// Simulations of the chemostat experiment

clear //clean all variables names
xdel(winsid())//close all figure windows

//////////////////////////////////////////////////
/////////Differential equations model////////////
////////////////////////////////////////////////

//ODE system
function Ksidot=DRUM(t, Ksi)  
       //kinetics
    rates = zeros(nb_reactions,1);

    //r_light
    eff_max =  params(1);
    rates(r_photo) = eff_max*intensite_max ;//          
    
    //r_g6p_from_gap
    k_g6p_from_gap = params(2);
    rates(r_g6p_from_gap) = k_g6p_from_gap*Ksi(GAP)/Ksi(BIOMf);
    
    //r_gap_from_g6p
    k_gap_from_g6p = params(3);
    rates(r_gap_from_g6p) = k_gap_from_g6p*Ksi(G6P)/Ksi(BIOMf);
    
    //r_carb
    k_carb = params(4); //h-1
    k_carb_r = params(5); //h-1
    rates(r_carb) = k_carb*Ksi(G6P)/Ksi(BIOMf) - k_carb_r*Ksi(CARB)/Ksi(BIOMf);
    
    //r_pa
    k_pa = params(6);//mmol-1.B.h-1
    k_par = params(7);//h-1
    rates(r_pa) = k_pa*Ksi(GAP)/Ksi(BIOMf)*Ksi(PEP)/Ksi(BIOMf) - k_par*Ksi(PA)/Ksi(BIOMf);

    //r_pep
    k_pep = params(8);//h-1
    k_pepr = params(9); //h-1
    rates(r_pep) = k_pep*Ksi(GAP)/Ksi(BIOMf) - k_pepr*Ksi(PEP)/Ksi(BIOMf);
    
    //r_biom
    k_biom = params(10);
    rates(r_biom) = k_biom*Ksi(PEP)/Ksi(BIOMf)*Ksi(G6P)/Ksi(BIOMf)*Ksi(NO3)*Ksi(PA)/Ksi(BIOMf);
    
    //r_exc
  
    select MODEL
    case 1 then 
     k_exc=params(11);
     rates(r_exc)=k_exc*Ksi(CARB)/Ksi(BIOMf);
    case 2 then
     k_exc=params(11);
     rates(r_exc)=k_exc*Ksi(G6P)/Ksi(BIOMf);    
    case 3 then
     k_exc=params(11);
     rates(r_exc)=k_exc*Ksi(PEP)/Ksi(BIOMf);   
    case 4 then
     k_exc=params(11);
     rates(r_exc)=k_exc*Ksi(GAP)/Ksi(BIOMf);
    end

    Ksidot(1:BIOM-1) = Stoich_mat(1:BIOM-1,:)*rates*Ksi(BIOMf) - D*Ksi(1:BIOM-1) ;
    Ksidot(BIOM) = nb_C(BIOMf)*Ksidot(BIOMf) + nb_C(CARB)*Ksidot(CARB) + nb_C(PA)*Ksidot(PA) + nb_C(GAP)*Ksidot(GAP) + nb_C(G6P)*Ksidot(G6P)+ nb_C(PEP)*Ksidot(PEP);
    Ksidot(NO3) = Stoich_mat(NO3,:)*rates*Ksi(BIOMf) - D*Ksi(NO3) + D*Sin;

endfunction  

//////////////////////////////////////////////////
///////////  Simulations ////////////////
//////////////////////////////////////////////////

t_step = 0.1 //h

DD=0.001:0.001:0.06;  // Dilution rates                 
nD=length(DD)


imax(1)=find(DD>0.054 ,1); // for each model, maximum dilution rate allowed
imax(2)=find(DD> 0.056,1);
imax(3)=find(DD>0.053 ,1);
imax(4)=find(DD>0.046 ,1);
imax(5)=find(DD>0.053 ,1);

qq=%nan*zeros(5,nD);
qg=%nan*zeros(5,nD);
ql=%nan*zeros(5,nD);
qd=%nan*zeros(5,nD);
qd2=%nan*zeros(5,nD);
exc=%nan*zeros(5,nD);
pho=%nan*zeros(5,nD);

for ii=0:4  // loop on the different models

MODEL=ii// 0: w/oEx* ; 1:Ex CARB; 2: Ex G6P; 3: Ex PEP; 4:Ex GAP; 

//params = [effmax k_g6p_from_gap k_gap_from_g6p k_carb k_carbr k_pa k_par k_pep k_pepr kbiom Qmax/k_ex Qmin];

select MODEL
case 0 then
      params=[ 0.0011011   62.582543   0.   13.472625   0.   141.40216   0.0431291   7707.7269   321.34625   317439.86];  
  case 1 then 
       params=[  0.0016079   6.0726315   0.0921596   13.609136   0.   0.783336   0.1202538   7964.8668   547.39568   2889.2591   0.0778283  ];
  case 2 then
      params=[  0.0021185   3.3175869   0.   46.222195   1.2162263   0.4128736   0.1167988   347.1997   121.54694   1335.7956   5.8980956 ];
  case 3 then         
    params=[    0.0045   569.20568   3.0372364   13376.282   376.41236   108.01982   0.090034   5169.46   0.   2028.0569   7.2933079  ] ;
  case 4 then
       params=[  0.0041455   0.5895331   2.6190586   402.83913   16.384763   0.0966455   0.1056239   26.229641   36.223695   1168.0651   3.4703291 ]; 
end
 
//////////////////////////////////////////////////
///////////Metabolic network//////////////////////
//////////////////////////////////////////////////

M_N = 14;
M_C = 12;

//metabolites
GAP = 1;
CARB = 2;
PEP = 3;
PA = 4;
G6P = 5;
NO3 = 6;
EPS=7;
ACE=8;
GLYC=9;
BIOMf = 10;
BIOM = 11;

//metabolite names
names_m(GAP)="GAP";
names_m(CARB)="CARB";
names_m(PEP)="PEP";
names_m(PA)="PA";
names_m(G6P)="G6P";
names_m(NO3)="NO3";
names_m(EPS)="EPS";
names_m(ACE)="ACE";
names_m(GLYC)="CLYC";
names_m(BIOMf)="BIOMf";
names_m(BIOM)="BIOM";

//number of Carbon atoms per molecule
nb_C(GAP)=3;
nb_C(CARB)=6;
nb_C(PEP)=3;
nb_C(PA)=36.2;
nb_C(G6P)=6;
nb_C(NO3)=0;
nb_C(BIOMf)=8.59;
nb_C(BIOM)=1;

nb_C(GLYC)=3;
nb_C(ACE)=2;
nb_C(EPS)=6;

//number of Azote atoms per molecule
nb_N(GAP)=0;
nb_N(CARB)=0;
nb_N(PEP)=0;
nb_N(PA)=0;
nb_N(G6P)=0;
nb_N(NO3)=1;
nb_N(BIOMf)=1.27;
nb_N(BIOM)=1;

//number of metabolites
nb_metabolites = size(names_m,1);

//reactions
r_photo = 1; //30 Light + 3 CO2 + 2 H2O + Pi --> GAP + 3 O2 (1)
r_g6p_from_gap = 2; //2 GAP + H2O --> Pi + G6P (2)
r_gap_from_g6p = 3; //ATP + G6P --> H + ADP + 2 GAP (3)
r_carb = 4; //G6P <--> CARB + Pi (4)
r_pa = 5; //34.382 H + 29.228 NADPH + 16.614 PEP + 13.46 NAD + 2.154 O2 + 2 ADP + 1 GAP <--> 1 PA + 2 ATP + 4.308 H2O + 13.46 NADH + 14.614 Pi + 16.614 CO2 + 29.228 NADP (5)
r_pep = 6; //1 Pi + 1 ADP + 1 GAP + 1 NAD --> 1 H + 1 ATP + 1 NADH + 1 H2O + 1 PEP (6)
r_biom = 7; //7.36572 O2 + 4.45869 H + 3.1265 PEP + 1.30971 NO3 + 1.14172 G6P + 0.1118 PA + 0.0303876 SO4 + 0.0025 Mg2 --> 1 Biomass + 4.22672 Pi + 6.00312 H2O + 11.6706 CO2 (7)
r_exc = 8; //excretion

//reactions names
names_r(r_photo)="Photosynthesis";
names_r(r_g6p_from_gap)="G6P to GAP";
names_r(r_gap_from_g6p)="GAP to G6P";
names_r(r_carb)="Carbohydrates synthesis";
names_r(r_pa)="Lipids synthesis";
names_r(r_pep)="Lower glycolysis";
names_r(r_biom)="Biomasse synthesis";
names_r(r_exc)="Excretion";

//Number of reactions
nb_reactions = size(names_r,1);

//stoichiometric matrix
Stoich_mat = zeros(nb_metabolites,nb_reactions);

//30 Light + 3 CO2 + 2 H2O + Pi --> GAP + 3 O2 (1)
Stoich_mat(GAP,r_photo)= 1;

//2 GAP + H2O --> Pi + G6P (2)
Stoich_mat(GAP,r_g6p_from_gap)= -2;
Stoich_mat(G6P,r_g6p_from_gap)= 1;

//ATP + G6P --> H + ADP + 2 GAP (3)
Stoich_mat(G6P,r_gap_from_g6p)= -1;
Stoich_mat(GAP,r_gap_from_g6p)= 2;

//G6P <--> CARB + Pi (4)
Stoich_mat(G6P,r_carb)= -1;
Stoich_mat(CARB,r_carb)= 1;

//34.382 H + 29.228 NADPH + 16.614 PEP + 13.46 NAD + 2.154 O2 + 2 ADP + 1 GAP <--> 1 PA + 2 ATP + 4.308 H2O + 13.46 NADH + 14.614 Pi + 16.614 CO2 + 29.228 NADP (5)
Stoich_mat(GAP,r_pa)= -1;
Stoich_mat(PEP,r_pa)= -16.614;
Stoich_mat(PA,r_pa)= 1;

//1 Pi + 1 ADP + 1 GAP + 1 NAD --> 1 H + 1 ATP + 1 NADH + 1 H2O + 1 PEP (6)
Stoich_mat(GAP,r_pep)= -1;
Stoich_mat(PEP,r_pep)= 1;

//7.36572 O2 + 4.45869 H + 3.1265 PEP + 1.30971 NO3 + 1.14172 G6P + 0.1118 PA + 0.0303876 SO4 + 0.0025 Mg2 --> 1 Biomass + 4.22672 Pi + 6.00312 H2O + 11.6706 CO2 (7)
Stoich_mat(G6P,r_biom)= -1.14172;
Stoich_mat(NO3,r_biom)= -1.30971;
Stoich_mat(PEP,r_biom)= -3.1265;
Stoich_mat(PA,r_biom)= -0.1118;
Stoich_mat(BIOMf,r_biom)= 1;

// Excretion
select MODEL
  case 1 then 
    Stoich_mat(CARB,r_exc)=-1;
    Stoich_mat(EPS,r_exc)=1;
  case 2 then
    Stoich_mat(G6P,r_exc)=-1;
    Stoich_mat(EPS,r_exc)=1;      
  case 3 then
    Stoich_mat(PEP,r_exc)=-1;
    Stoich_mat(ACE,r_exc)=1;      
  case 4 then
    Stoich_mat(GAP,r_exc)=-1;
    Stoich_mat(GLYC,r_exc)=1;    
end


//////////////////////////////////////////////////
///////////Initial conditions/////////////////////
//////////////////////////////////////////////////
Ksi0=zeros(nb_metabolites,1);
Ksi0(BIOM) = 28.5/M_C;
Ksi0(GAP) = 0.0006/nb_C(GAP);
Ksi0(CARB) = 0.37*Ksi0(BIOM)/nb_C(CARB);
Ksi0(PEP) = 0.004/nb_C(PEP);
Ksi0(PA) = 0.20*Ksi0(BIOM)/nb_C(PA);
Ksi0(G6P) = 0.03/nb_C(G6P);
Ksi0(NO3) = 1.6/(nb_N(NO3)*M_N);
Ksi0(BIOMf) = (Ksi0(BIOM)-Ksi0(PA)*nb_C(PA)-Ksi0(CARB)*nb_C(CARB))/(nb_C(BIOMf));

Ksi0(EPS)=0;
Ksi0(ACE)=0;
Ksi0(GLYC)=0;

for i=1:imax(ii+1) // loop on the dilution rates
                
//Solve the ODE system
t_start = 0;
t_end = 7000;  // h
t = t_start:t_step:t_end;

D = DD(i);
Sin = 0.287;
intensite_max = 430;

Ksis=ode(Ksi0,t_start,t,list(DRUM))

//Compute annex variables to ODE system
qq(ii+1,i)=(Ksis(BIOMf,$)*nb_N(BIOMf)*M_N) ./ ( Ksis(BIOM,$)*nb_C(BIOM)*M_C ) ;  // functional biomass quota at equilibrium
qg(ii+1,i)=nb_C(CARB)* Ksis(CARB,$)./(Ksis(BIOM,$)*nb_C(BIOM));  // carbohydrate quota at equilibrium
ql(ii+1,i)=nb_C(PA)*Ksis(PA,$)./(Ksis(BIOM,$)*nb_C(BIOM));   // neutral lipid quota at equilibrium

eff_max = params(1);

select MODEL
case 1 then 
    k_exc=params(11);
    exc(ii+1,i)=6* k_exc*Ksis(CARB,$)./Ksis(BIOMf,$); // excretion rate
    pho(ii+1,i)=  (3*eff_max'.*intensite_max); // photosynthetic rate
case 2 then
    k_exc=params(11);
    exc(ii+1,i)=6*k_exc*Ksis(G6P,$)./Ksis(BIOMf,$)
    pho(ii+1,i)=  (3*eff_max'.*intensite_max);
case 3 then
    k_exc=params(11);
    exc(ii+1,i)=3*k_exc*Ksis(PEP,$)./Ksis(BIOMf,$)
    pho(ii+1,i)=  (3*eff_max'.*intensite_max);
case 4 then
    k_exc=params(11);
    exc(ii+1,i)=3*k_exc*Ksis(GAP,$)./Ksis(BIOMf,$)
    pho(ii+1,i)=  (3*eff_max'.*intensite_max);
end

end
end

//////////////   Plot //////////////////////////


// Experimental data from Lacour et al.
DDE=[ .4 .6 .8 1]/24; // Dilution rates

qnE1=[ .06 .066 .069 .092];  // functional biomass quota at equilibrium
qlE1=[ .102 .104 .14 .19]; // neutral lipid quota at equilibrium
qgE1=[ 0.57 0.58 0.44 0.315]; // carbohydrate quota at equilibrium

qnE2=[ 0.051 0.059 0.068 0.0975];
qlE2=[ %nan %nan %nan 0.175];
qgE2=[ 0.6 0.62 0.53 0.315];

// Parameters for the Droop model
mub= 0.072
q0=   0.040

qqq=q0:0.001:0.12;

figure("Figure_name", "ValidationA","BackgroundColor",[1,1,1]);
subplot(121)
plot2d(qq(1,:),DD*24,1)
a9=gca();
a9.children(1).children(1).line_style=7;
a9.children(1).children(1).thickness = 2;
plot2d(qqq,mub*24*(1-q0 ./ qqq),1)
a9=gca();
a9.children(1).children(1).line_style=1;
a9.children(1).children(1).thickness = 1;
plot(qnE1,DDE*24,'kv')
plot(qnE2,DDE*24,'k^')

replot([0 0 0.12 1.4])
xlabel('N/C')
ylabel('Specific growth rate µ (/d)')
title('Model without excretion')
a9=gca();
a9.box = "on";

subplot(122)
plot2d(qq(2,:),DD*24,2)
a9=gca();
a9.children(1).children(1).line_style=1;
a9.children(1).children(1).thickness = 2;
plot2d(qq(3,:),DD*24,3)
a9=gca();
a9.children(1).children(1).line_style=2;
a9.children(1).children(1).thickness = 2;
plot2d(qq(4,:),DD*24,4)
a9=gca();
a9.children(1).children(1).line_style=3;
a9.children(1).children(1).thickness = 2;
plot2d(qq(5,:),DD*24,5)
a9=gca();
a9.children(1).children(1).line_style=4;
a9.children(1).children(1).thickness = 2;
plot2d(qqq,mub*24*(1-q0 ./ qqq),1)
a9=gca();
a9.children(1).children(1).line_style=1;
a9.children(1).children(1).thickness = 1;
plot(qnE1,DDE*24,'kv')
plot(qnE2,DDE*24,'k^')

replot([0 0 0.12 1.4])
xlabel('N/C')
ylabel('Specific growth rate µ (/d)')
title('Model with excretion')
a9=gca();
a9.box = "on";

//////////////////////////////////////////////////////////


DDp=[ .4 .6 .8 1];

figure("Figure_name", "ValidationB","BackgroundColor",[1,1,1]);
subplot(231)
 y=[1-(qlE1+qgE1);qlE1;qgE1]';
bar(DDp,y,'stacked');
plot(0.4,0)
ylabel('Mass fraction')
xlabel('Specific growth rate (/d)')
title('Experiments')

qll=interp1(DD,ql(1,:),DDE,'linear','extrap');
qgg=interp1(DD,qg(1,:),DDE,'linear','extrap');

subplot(232)
 y=[1-(qll+qgg);qll;qgg]';
bar(DDp,y,'stacked');
plot(0.4,0)
ylabel('Mass fraction')
xlabel('Specific growth rate (/d)')
title('Model w/oEx*')

qll=interp1(DD,ql(2,:),DDE,'linear','extrap');
qgg=interp1(DD,qg(2,:),DDE,'linear','extrap');

subplot(233)
 y=[1-(qll+qgg);qll;qgg]';
bar(DDp,y,'stacked');
 plot(DDp,interp1(DD,exc(2,:)./pho(2,:),DDE,'linear','extrap'),'wd','markersize',12)
ylabel('Mass fraction')
xlabel('Specific growth rate (/d)')
title('Model ExCARB ')

qll=interp1(DD,ql(3,:),DDE,'linear','extrap');
qgg=interp1(DD,qg(3,:),DDE,'linear','extrap');

subplot(234)
 y=[1-(qll+qgg);qll;qgg]';
bar(DDp,y,'stacked');
plot(DDp,interp1(DD,exc(3,:)./pho(3,:),DDE,'linear','extrap'),'wd','markersize',12)
ylabel('Mass fraction')
xlabel('Specific growth rate (/d)')
title('Model ExG6P')

qll=interp1(DD,ql(4,:),DDE,'linear','extrap');
qgg=interp1(DD,qg(4,:),DDE,'linear','extrap');

subplot(235)
 y=[1-(qll+qgg);qll;qgg]';
bar(DDp,y,'stacked');
plot(DDp,interp1(DD,exc(4,:)./pho(4,:),DDE,'linear','extrap'),'wd','markersize',12)
ylabel('Mass fraction')
xlabel('Specific growth rate (/d)')
title('Model ExPEP')

qll=interp1(DD,ql(5,:),DDE,'linear','extrap');
qgg=interp1(DD,qg(5,:),DDE,'linear','extrap');

subplot(236)
 y=[1-(qll+qgg);qll;qgg]';
bar(DDp,y,'stacked');
plot(DDp,interp1(DD,exc(5,:)./pho(5,:),DDE,'linear','extrap'),'wd','markersize',12)
ylabel('Mass fraction')
xlabel('Specific growth rate (/d)')
title('Model ExGAP')






